property debug_me : false

property script_author : "Jonathan Nathan"
property script_name : "keylister"
property script_version : "1.0.1"
property script_release_date : "January 31, 2007"
property script_copyright : " 2004, 2007 Jonathan Nathan. All Rights Reserved."

property script_url : "http://www.jonn8.com/"
property script_email : "mailto:Jonathan%20Nathan%20%3Cjonn8%40mac.com%3E?subject=Re:keylister"

property stop_progress : false
property first_run : true
property new_data_source : missing value

on will finish launching the_object
	set object_name to name of the_object as Unicode text
	my update_status("will finish launching " & object_name)
	my register_settings()
	my read_settings()
end will finish launching

on launched the_object
	my update_status("launched " & (name of the_object as Unicode text))
	activate
	show window "main"
	if first_run then display window "license" attached to window "main"
end launched

on should quit after last window closed the_object
	return true
end should quit after last window closed

on awake from nib the_object
	set object_name to name of the_object as Unicode text
	my update_status("awake from nib " & object_name)
	if object_name = "main" then
		set new_data_source to make new data source at end of data sources
		tell the_object
			if first_run then center
			set the_table to table view 1 of scroll view 1
			set data source of the_table to new_data_source
			tell new_data_source
				make new data column at end with properties {name:"name"}
				make new data column at end with properties {name:"account"}
				make new data column at end with properties {name:"password"}
			end tell
		end tell
		
	else if object_name = "about" then
		tell the_object
			center
			set contents of text field "title" to (script_name & " v" & script_version)
			set contents of text field "author" to ("by " & script_author)
			set contents of text field "release" to ("Released " & script_release_date)
			set contents of text field "copyright" to (script_copyright)
		end tell
		
	end if
end awake from nib

on activated the_object
	set object_name to name of the_object as Unicode text
	my update_status("activated " & object_name)
end activated

on choose menu item the_object
	set object_name to name of the_object as Unicode text
	my update_status("choose menu item " & object_name)
	
	if object_name = "about" then
		display window "about" attached to window "main"
		
	else if object_name = "prefs" then
		display window "prefs" attached to window "main"
		
	else if object_name = "quit" then
		quit
		
	end if
end choose menu item

on clicked the_object
	set object_name to name of the_object as Unicode text
	my update_status("clicked " & object_name)
	
	if object_name = "decline" then
		close panel window "license"
		quit
		
	else if object_name = "accept" then
		set first_run to false
		my write_settings()
		close panel window "license"
		
	else if object_name = "dialog_cancel" then
		set the_win to name of window of the_object
		close panel window the_win
		
	else if object_name = "dialog_ok" then
		set the_win to name of window of the_object
		close panel window the_win
		
	else if object_name = "stop_progress" then
		set my stop_progress to false
		
	else if object_name = "about_webpage" then
		open location script_url
		
	else if object_name = "about_email" then
		open location script_email
		
	end if
end clicked

on action the_object
	set object_name to name of the_object as Unicode text
	my update_status("action " & object_name)
	
	if object_name = "the_key" then
		my get_key(string value of the_object)
		
	end if
end action

on will quit the_object
	my quit_prep()
	return true
end will quit

on quit_prep()
	set panel_wins to name of (windows whose class = panel and visible = true)
	repeat with i from 1 to (length of panel_wins)
		try
			close panel window (item i of panel_wins)
		end try
	end repeat
	my write_settings()
end quit_prep

on progress_show(super_message, the_message)
	set stop_progress to true
	tell window "progress"
		set indeterminate of progress indicator "progress" to true
		set uses threaded animation of progress indicator "progress" to true
		tell progress indicator "progress" to start
		set contents of text field "progress_label_super" to super_message
		set contents of text field "progress_label" to the_message
		update
	end tell
	if visible of window "progress" = false then display window "progress" attached to window "main"
end progress_show

on progress_update(iteration, total_count, super_message, the_message)
	tell window "progress"
		if iteration = 1 then
			tell progress indicator "progress" to stop
			set indeterminate of progress indicator "progress" to false
		end if
		set contents of text field "progress_label_super" to super_message
		set contents of text field "progress_label" to the_message
		set maximum value of progress indicator "progress" to total_count
		set content of progress indicator "progress" to iteration
		update
	end tell
end progress_update

on progress_hide()
	close panel window "progress"
	tell window "progress"
		set contents of text field "progress_label_super" to ""
		set contents of text field "progress_label" to ""
		set content of progress indicator "progress" to 0
		tell progress indicator "progress" to stop
		set indeterminate of progress indicator "progress" to false
		update
	end tell
end progress_hide

on display_error(the_title, the_error)
	activate
	beep
	my update_status("Error: " & the_error)
	display alert the_title as critical message the_error default button "OK" attached to window "main"
	return false
end display_error

on register_settings()
	my update_status("registering settings...")
	tell user defaults
		make new default entry at end of default entries with properties {name:"first_run", contents:first_run}
		register
	end tell
	my update_status("settings registered")
end register_settings

on read_settings()
	my update_status("reading settings...")
	tell user defaults
		set first_run to contents of default entry "first_run"
	end tell
	my update_status("settings read")
end read_settings

on write_settings()
	my update_status("writing settings...")
	tell user defaults
		set contents of default entry "first_run" to first_run
	end tell
	my update_status("settings written")
end write_settings

on update_status(the_message)
	if debug_me then log the_message
end update_status

on get_key(the_key)
	if the_key is in {"", " "} then return false
	set current_user to do shell script "whoami"
	my progress_show("Searching Keychain", "Searching keychain " & current_user & " for " & the_key & "...")
	set found_keys to {}
	tell application "Keychain Scripting"
		tell keychain current_user
			repeat with this_key in (get keys)
				if not stop_progress then exit repeat
				if (name of this_key) contains the_key then set end of found_keys to {this_key's name, this_key's account, this_key's password}
			end repeat
		end tell
	end tell
	my progress_hide()
	if found_keys = {} then
		my display_error("Get Key Error", "There was no match for " & the_key & ". Try again.")
	else
		tell new_data_source
			try
				delete data rows
			end try
			append with found_keys
		end tell
	end if
end get_key

